@@ -8,7 +8,7 @@ use SVK::Target;
 use Pod::Simple::Text ();
 use Pod::Simple::SimpleTree ();
 use File::Find ();
-use SVK::Util qw( get_prompt abs_path is_uri catdir bsd_glob $SEP IS_WIN32 HAS_SVN_MIRROR );
+use SVK::Util qw( get_prompt abs2rel abs_path is_uri catdir bsd_glob $SEP IS_WIN32 HAS_SVN_MIRROR );
 use SVK::I18N;
 
 =head1 NAME
@@ -182,11 +182,19 @@ additional %opt getopt options.
 
 =cut
 
+use constant opt_recursive => undef;
+
 sub getopt {
     my ($self, $argv, %opt) = @_;
     local *ARGV = $argv;
+    my $recursive = $self->opt_recursive;
+    my $toggle = 0;
+    $opt{$recursive ? 'N|non-recursive' : 'R|recursive'} = \$toggle
+	if defined $recursive;
     die loc ("Unknown options.\n")
 	unless GetOptions (%opt, $self->_opt_map ($self->options));
+    $self->{recursive} = ($recursive + $toggle) % 2
+	if defined $recursive;
 }
 
 =head2 Instance Methods
@@ -204,6 +212,11 @@ C<$self>.
 Subclasses should override this to add their own options.  Defaults to
 an empty list.
 
+=head3 opt_recursive
+
+Defines if the command needs the recursive flag and its default.  The
+value will be stored in C<recursive>.
+
 =cut
 
 sub options { () }
@@ -289,6 +302,18 @@ sub arg_condensed {
     s{[/\Q$SEP\E]$}{}o for @arg; # XXX band-aid
 
     my ($report, $copath, @targets )= $self->{xd}->condense (@arg);
+
+    if ($self->{recursive}) {
+	# remove redundant targets when doing recurisve
+	# if have '' in targets then it means everything
+	my @newtarget = @targets;
+	for my $anchor (sort {length $a <=> length $b} @targets) {
+	    @newtarget = grep { length $anchor ? $_ eq $anchor || index ($_, "$anchor/") != 0
+				               : 0} @newtarget;
+	}
+	@targets = @newtarget;
+    }
+
     my ($repospath, $path, undef, $cinfo, $repos) = $self->{xd}->find_repos_from_co ($copath, 1);
     my $target = SVK::Target->new
 	( repos => $repos,
@@ -322,8 +347,10 @@ sub arg_uri_maybe {
     is_uri($arg) or return $self->arg_depotpath($arg);
     HAS_SVN_MIRROR or die loc("cannot load SVN::Mirror");
 
+    $arg =~ s{/?$}{/}; # add a trailing slash at the end
+
     require URI;
-    my $uri = URI->new("$arg/")->canonical or die loc("%1 is not a valid URI.\n", $arg);
+    my $uri = URI->new($arg)->canonical or die loc("%1 is not a valid URI.\n", $arg);
     my $map = $self->{xd}{depotmap};
     foreach my $depot (sort keys %$map) {
         local $@;
@@ -726,6 +753,36 @@ sub rebless {
     return $self->command($command, $args, 1);
 }
 
+sub find_checkout_anchor {
+    my ($self, $target, $track_merge, $track_sync) = @_;
+
+    my $entry = $self->{xd}{checkout}->get ($target->{copath});
+    my $anchor_target = $self->arg_depotpath ($entry->{depotpath});
+
+    return ($anchor_target, undef) unless $track_merge;
+
+    my @rel_path = split(
+        '/',
+        abs2rel ($target->{path}, $anchor_target->{path}, undef, '/')
+    );
+
+    my $copied_from;
+    while (!$copied_from) {
+        $copied_from = $anchor_target->copied_from ($track_sync);
+
+        if ($copied_from) {
+            return ($anchor_target, $copied_from);
+        }
+        elsif (@rel_path) {
+            $anchor_target->descend (shift (@rel_path));
+        }
+        else {
+            return ($self->arg_depotpath ($entry->{depotpath}), undef);
+        }
+    }
+}
+
+
 1;
 
 __DATA__
