@@ -4,7 +4,7 @@ our $VERSION = $SVK::VERSION;
 use base qw( SVK::Command );
 use SVK::XD;
 use SVK::I18N;
-use SVK::Util qw( $EOL );
+use SVK::Util qw( traverse_history );
 use Algorithm::Annotate;
 
 sub options {
@@ -23,30 +23,29 @@ sub lock { $_[0]->lock_none }
 sub run {
     my ($self, $target) = @_;
 
-    my $pool = SVN::Pool->new_default_sub;
     my $fs = $target->{repos}->fs;
-    my $root = $fs->revision_root ($target->{revision});
     my $ann = Algorithm::Annotate->new;
     my @revs;
 
-    my $hist = $root->node_history ($target->{path});
-    my $spool = SVN::Pool->new_default ($pool);
-    $self->{cross} ||= 0;
-    while ($hist = $hist->prev($self->{cross})) {
-	my ($path, $rev) = $hist->location;
-	unshift @revs, [$path, $rev];
-	$spool->clear;
-    }
+    traverse_history (
+        root     => $fs->revision_root ($target->{revision}),
+        path     => $target->{path},
+        cross    => $self->{cross},
+        callback => sub {
+            my ($path, $rev) = @_;
+            unshift @revs, [ $path, $rev ];
+            1;
+        }
+    );
 
     print loc("Annotations for %1 (%2 active revisions):\n", $target->{path}, scalar @revs);
     print '*' x 16;
     print "\n";
     for (@revs) {
-	$spool->clear;
 	local $/;
 	my ($path, $rev) = @$_;
 	my $content = $fs->revision_root ($rev)->file_contents ($path);
-	$content = [split /$EOL/o, <$content>];
+	$content = [split /\015?\012|\015/, <$content>];
 	no warnings 'uninitialized';
 	$ann->add ( sprintf("%6s\t(%8s %10s):\t\t", $rev,
 			    $fs->revision_prop ($rev, 'svn:author'),
@@ -58,15 +57,18 @@ sub run {
     if ($target->{copath}) {
 	$final = SVK::XD::get_fh ($target->root ($self->{xd}), '<', $target->{path}, $target->{copath});
 	local $/;
-	$ann->add ( "\t(working copy): \t\t", [split /$EOL/o, <$final>]);
-	seek $final, 0, 0;
+	$final = [split /\015?\012|\015/, <$final>];
+	$ann->add ( "\t(working copy): \t\t", $final );
     }
     else {
+	local $/;
 	$final = $fs->revision_root($revs[-1][1])->file_contents($revs[-1][0]);
+	$final = [split /\015?\012|\015/, <$final>];
     }
+
     my $result = $ann->result;
     while (my $info = shift @$result) {
-	print $info.<$final>;
+	print $info, shift(@$final), "\n";
     }
 
 }
@@ -82,6 +84,7 @@ SVK::Command::Annotate - Display per-line revision and author info
 =head1 SYNOPSIS
 
  annotate FILE
+ annotate DEPOTPATH[@rev]
 
 =head1 OPTIONS
 
