@@ -5,13 +5,14 @@ our $VERSION = $SVK::VERSION;
 use base qw( SVK::Command::Update );
 use SVK::XD;
 use SVK::I18N;
-use SVK::Util qw( abs_path move_path is_empty_path $SEP );
-use File::Spec;
+use SVK::Util qw( get_anchor abs_path move_path splitdir $SEP );
+use File::Path;
 
 sub options {
     ($_[0]->SUPER::options,
      'l|list' => 'list',
      'd|delete|detach' => 'detach',
+     'export' => 'export',
      'relocate' => 'relocate');
 }
 
@@ -27,7 +28,8 @@ sub parse_arg {
     die loc("don't know where to checkout %1\n", $arg[0]) unless $arg[1] || $depotpath->{path} ne '/';
 
     $arg[1] =~ s|/$|| if $arg[1];
-    $arg[1] ||= (File::Spec->splitdir($depotpath->{path}))[-1];
+    $arg[1] = (splitdir($depotpath->{path}))[-1]
+        if !defined($arg[1]) or $arg[1] =~ /^\.?$/;
 
     return ($depotpath, $arg[1]);
 }
@@ -120,8 +122,21 @@ sub _do_relocate {
 sub _do_checkout {
     my ($self, $target, $report) = @_;
 
+    if (-e $report) {
+	die loc("Checkout path %1 already exists.\n", $report);
+    }
+    else {
+	# Cwd is annoying, returning undef for paths whose parent.
+	# we can't just mkdir -p $report because it might be a file,
+	# so let C::Update take care about it.
+	my ($anchor) = get_anchor (0, $report);
+	if (length $anchor && !-e $anchor) {
+	    mkpath [$anchor] or
+		die loc ("Can't create checkout path %1: $!\n", $anchor);
+	}
+    }
+
     my $copath = abs_path ($report);
-    die loc("Checkout path %1 already exists.\n", $copath) if -e $copath;
 
     my ($entry, @where) = $self->{xd}{checkout}->get ($copath);
     die loc("Overlapping checkout path is not supported (%1); use 'svk checkout --detach' to remove it first.\n", $where[0])
@@ -137,8 +152,11 @@ sub _do_checkout {
 					       });
     $self->{rev} = $target->{repos}->fs->youngest_rev unless defined $self->{rev};
 
-    return $self->SUPER::run ($target->new (report => $report,
-					    copath => $copath));
+    $self->SUPER::run ($target->new (report => $report,
+				     copath => $copath));
+    $self->_do_detach ($copath)
+	if $self->{export};
+    return;
 }
 
 sub _find_copath {
@@ -177,6 +195,8 @@ SVK::Command::Checkout - Checkout the depotpath
  -r [--revision] arg    : act on revision ARG instead of the head revision
  -l [--list]            : list checkout paths
  -d [--detach]          : mark a path as no longer checked out
+ -q [--quiet]           : quiet mode
+ --export               : export mode; checkout a detached copy
  --relocate             : relocate the checkout to another path
 
 =head1 AUTHORS
