@@ -8,6 +8,10 @@ use SVK::XD;
 sub options {
     ('f|from=s'         => 'from_path',
      'l|lump'           => 'lump',
+     'C|check-only'     => 'check_only',
+     'S|sign'	        => 'sign',
+     'P|patch=s'        => 'patch',
+     'verbatim'		=> 'verbatim',
     );
 }
 
@@ -21,7 +25,9 @@ sub parse_arg {
         unshift @arg, $self->{from_path};
     }
 
-    if ($self->{lump}) {
+    # "svk push -P" has the same effect as "svk push -l",
+    # because incremental patches is not yet implemented.
+    if ($self->{lump} or $self->{patch}) {
         $self->{log}++;
         $self->{message} = '';
         delete $self->{incremental};
@@ -49,6 +55,10 @@ SVK::Command::Push - Move changes into another repository
 
  -f [--from] arg        : push from the specified path
  -l [--lump]            : merge everything into a single commit log
+ -C [--check-only]      : try operation but make no changes
+ -P [--patch] arg       : instead of commit, save this change as a patch
+ -S [--sign]            : sign this change
+ --verbatim             : verbatim merge log without indents and header
 
 =head1 AUTHORS
 
