@@ -372,7 +372,8 @@ sub condense {
 	    $schedule = $cinfo->{'.schedule'} || '';
 	}
     }
-    return ($report, $anchor, map { abs2rel($_, $anchor) } grep {$_ ne $anchor} @targets);
+    return ($report, $anchor, $#targets == 0 && $targets[0] eq $anchor ? ()
+	    : map { abs2rel($_, $anchor) } @targets);
 }
 
 sub xdroot {
@@ -607,9 +608,11 @@ sub do_delete {
 	     push @deleted, $cpath;
 	 }, @paths) if @paths;
 
+    # need to use undef to avoid the $SEP at the beginning on empty report.
+    my $report = length ($arg{report}) ? $arg{report} : undef;
     for (@deleted) {
-	my $rpath = abs2rel($_, $arg{copath} => $arg{report});
-	print "D   $rpath\n" unless $arg{quiet};
+	print "D   ".abs2rel($_, $arg{copath} => $report)."\n"
+	    unless $arg{quiet};
 	$self->{checkout}->store ($_, {'.schedule' => 'delete'});
     }
 
@@ -978,10 +981,20 @@ sub _delta_dir {
 	    $targets->{$file} = undef;
 	}
     }
+    my $thisdir;
+    if ($targets) {
+	if (exists $targets->{''}) {
+	    delete $targets->{''};
+	    $thisdir = 1;
+	}
+    }
+    else {
+	$thisdir = 1;
+    }
     # don't use depth when we are still traversing through targets
     my $descend = defined $targets || !(defined $arg{depth} && $arg{depth} == 0);
     $arg{cb_conflict}->($arg{editor}, $arg{entry}, $arg{baton})
-	if $arg{cb_conflict} && $cinfo->{'.conflict'};
+	if $thisdir && $arg{cb_conflict} && $cinfo->{'.conflict'};
 
     return if $self->_node_deleted_or_absent (%arg, pool => $pool);
     $arg{base} = 0 if $schedule eq 'replace';
@@ -1026,7 +1039,6 @@ sub _delta_dir {
 	my $copath = SVK::Target->copath ($arg{copath}, $entry);
 	my $ccinfo = $self->{checkout}->get ($copath);
 	next if $unchanged && !$ccinfo->{'.schedule'} && !$ccinfo->{'.conflict'};
-	# XXX: save this stat() result for node_delete_or_absent
 	lstat ($copath);
 	my $type = -e _ ? (-d _ and not is_symlink) ? 'directory' : 'file'
 	                : '';
@@ -1115,13 +1127,13 @@ sub _delta_dir {
 
     }
 
-    if (defined $targets) {
-	print loc ("Unknown target: %1.\n", $_) for sort keys %$targets;
-    }
-    else {
+    if ($thisdir) {
 	$arg{editor}->change_dir_prop ($baton, $_, ref ($newprops->{$_}) ? undef : $newprops->{$_}, $pool)
 	    for sort keys %$newprops;
     }
+    if (defined $targets) {
+	print loc ("Unknown target: %1.\n", $_) for sort keys %$targets;
+    }
 
     $arg{editor}->close_directory ($baton, $pool)
 	unless $arg{root};
